module Parser(
  ASTNode (..),
  recognizeAST
  )where

import Lexer

data ASTNode = ASTLiteralNumber Double
             | ASTLiteralString String
             | ASTLiteralChar Char
             | ASTSymbol String
             | ASTQuote ASTNode
             | AST [ASTNode]
             | ASTErr String
  deriving (Eq, Show)

recognizeASTNode :: [Token] -> ([Token], ASTNode)
recognizeASTNode [] = ([], ASTErr "Unexpected EOF")
recognizeASTNode ((TokNumber x):xs) = (xs, ASTLiteralNumber x)
recognizeASTNode ((TokStr x):xs) = (xs, ASTLiteralString x)
recognizeASTNode ((TokCharacter x):xs) = (xs, ASTLiteralChar x)
recognizeASTNode ((TokSymbol x):xs) = (xs, ASTSymbol x)
recognizeASTNode (TokQuote:xs) = case recognizeASTNode xs of (ys, n) -> (ys, ASTQuote n)
recognizeASTNode (TokLPar:xs) = case recognizeSubAST xs of (ys, n) -> (ys, AST n)
recognizeASTNode (TokRPar:xs) = (xs, ASTErr "unreachable error") -- should never be called on TokRPar

recognizeSubAST :: [Token] -> ([Token], [ASTNode])
recognizeSubAST [] = ([], [ASTErr "Unexpected EOF"])
recognizeSubAST (x:xs) | x == TokRPar = (xs, [])
                       | otherwise = case recognizeASTNode (x:xs) of
                                       (ys, node) ->
                                         case recognizeSubAST ys of
                                           (zs, ast) -> (zs, node:ast)

recognizeAST' :: [Token] -> ([Token], [ASTNode])
recognizeAST' [] = ([], [])
recognizeAST' (x:xs) | x == TokRPar = (xs, [ASTErr "Unexpected ')'"])
                     | otherwise = case recognizeASTNode (x:xs) of
                                     (ys, node) ->
                                       case recognizeAST' ys of
                                         (zs, ast) -> (zs, node:ast)

recognizeAST :: [Token] -> [ASTNode]
recognizeAST xs = case recognizeAST' xs of
  (_, ys) -> ys
